/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __MODEL_FILTER_ECHO_BASE_H_
#define __MODEL_FILTER_ECHO_BASE_H_

#include "gdamfilter.h"
#include "gdaminfomanager.h"

typedef struct _GdamEchoBase GdamEchoBase;
typedef struct _GdamEchoBaseClass GdamEchoBaseClass;

GtkType gdam_echo_base_get_type();
#define GDAM_TYPE_ECHO_BASE			(gdam_echo_base_get_type ())
#define GDAM_ECHO_BASE(obj)			(GTK_CHECK_CAST ((obj), GDAM_TYPE_ECHO_BASE, GdamEchoBase))
#define GDAM_ECHO_BASE_CLASS(klass)		(GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_ECHO_BASE, GdamEchoBaseClass))
#define GDAM_IS_ECHO_BASE(obj)		(GTK_CHECK_TYPE ((obj), GDAM_TYPE_ECHO_BASE))
#define GDAM_IS_ECHO_BASE_CLASS(klass)	(GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_ECHO_BASE))

struct _GdamEchoBaseClass {
	GdamFilterClass		filter_class;

	gboolean                (*refresh_echo_array)  (GdamEchoBase* e_base);
};

struct _GdamEchoBase 
{
	GdamFilter		filter;
	  /* The number of echoes contained in the echoes array.  */
	int 			num_echoes;
	  /* num_echoes*2 doubles correspond volume/position pairs. */
	gdouble*		echoes;
	  /* Should we apply reverb */
	gboolean		do_reverb;
	gboolean		making_request;
	gdouble			passthrough_volume;
	gdouble			wet_volume;
	  /* Preserve server-side buffer with a zero-volume echo. */
	gdouble			dummy_beat;
	GdamRequest*		pending_request;
	gboolean		ignore_reverb;
	gboolean		ignore_passthrough;
	int		        passthrough_muted;
};

gboolean gdam_echo_base_refresh_echo_array    (GdamEchoBase*      echo_base);
void     gdam_echo_base_update                (GdamEchoBase*      echo_base);

#endif
