/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
/*
 * A view on the builder, which is just a filter-stack + underlying component.
 */
#ifndef __GDAMBUILDER_SKIN__H_
#define __GDAMBUILDER_SKIN__H_

#include "gdamskin.h"
#include <gdam/component/gdambuilder.h>


typedef struct _GdamBuilderSkinClass GdamBuilderSkinClass;
typedef struct _GdamBuilderSkin GdamBuilderSkin;


GtkType gdam_builder_skin_get_type();
#define GDAM_TYPE_BUILDER_SKIN              (gdam_builder_skin_get_type ())
#define GDAM_BUILDER_SKIN(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_BUILDER_SKIN, GdamBuilderSkin))
#define GDAM_BUILDER_SKIN_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_BUILDER_SKIN, GdamBuilderSkinClass))
#define GDAM_IS_BUILDER_SKIN(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_BUILDER_SKIN))
#define GDAM_IS_BUILDER_SKIN_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_BUILDER_SKIN))

struct _GdamBuilderSkinClass {
	GdamSkinClass		skin_class;
};

/* A skin class, and a NULL-terminated list of options. */
void gdam_builder_skin_class_add_extra (const char* skin_class_name,
					const char* skin_filename,
					const char* label_name,
                                        const char** options);

struct _GdamBuilderSkin {
	GdamSkin		base;
	GtkContainer*           child_container;
	GdamSkin*		child_skin;
	GtkBox*			filterbox;
	GtkBox*			button_box;
	GHashTable*		filter_boxes;
};

typedef struct _NudgeInfo {
	GdamBuilder* 		builder;
	float			amount;
} NudgeInfo;

gboolean gdam_builder_skin_set_child(GdamBuilderSkin* 	skin,
				     const char* 	skin_type_name, 
				     const char* 	skin_filename,
				     int                num_options,
				     char**             options);

gboolean gdam_builder_skin_append_canned
                                    (GdamBuilderSkin*   skin,
                                     const char*        skin_xml_file);

gboolean gdam_builder_skin_move_up  (GdamBuilderSkin* 	skin,
				     GdamSkin*          filter_skin);
gboolean gdam_builder_skin_move_down(GdamBuilderSkin* 	skin,
				     GdamSkin*          filter_skin);

#endif
